@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 18:45:55 1995
  * Project:     INN (innfeed)
  * File:        article.c
- * RCSId:       $Id: article.c,v 1.15 1996/06/05 13:55:16 brister Exp brister $
+ * RCSId:       $Id: article.c,v 1.16 1996/08/12 09:07:56 brister Exp $
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
  *              Permission to use, copy, modify, and distribute this
@@ -37,7 +37,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: article.c,v 1.15 1996/06/05 13:55:16 brister Exp brister $" ;
+static const char *rcsid = "$Id: article.c,v 1.16 1996/08/12 09:07:56 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -249,8 +249,7 @@ Article newArticle (const char *filename, const char *msgid)
   else
     {
       if (strcmp (filename,newArt->fname) != 0)
-        die ("Two different file names for the same article: %s and %s",
-             filename, newArt->fname) ;
+        syslog (LOG_ERR, DOUBLE_NAME, filename, newArt->fname) ;
       
       newArt->refCount++ ;
       dprintf (2,"Reusing exiting article for %s\nx",msgid) ;
