@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:34:23 1995
  * Project:     INN (innfeed)
  * File:        config.h
- * RCSId:       $Id: config.h,v 1.11 1996/06/05 13:57:57 brister Exp brister $
+ * RCSId:       $Id: config.h,v 1.14 1996/12/07 01:29:51 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -45,6 +45,9 @@ file.
   /* Go edit sysconfig.h for platform differences */
 #include "sysconfig.h"
 
+/* set this to 1 if you want innfeed.status to be wrapped in HTML code. */
+#define GEN_HTML 0
+
   /**********************************************************************/
   /*                     Application specific defines                   */
   /**********************************************************************/
@@ -56,6 +59,9 @@ file.
   /* the path to the run-time config file */
 #define CONFIG_FILE "/var/tmp/innfeed/innfeed.conf"
 
+  /* where to scribble the process id */
+#define PID_FILE "/var/tmp/innfeed/innfeed.pid"
+
   /* the path to a log file where stdout and stderr will go if they're
      pointing at /dev/null on program start. This will catch any fatal
      messages that syslog didn't. Can override with '-l' option. If stderr
@@ -168,6 +174,9 @@ file.
      probably got a memory leak so we print a snapshot and dump core. */
 #define SOFT_ARTICLE_BYTE_LIMIT (1024 * 1024 * 10) /* 10MB */
 
+/* define SELECT_RATIO to the number of times through the main loop before
+   checking on the fd from inn again.... */
+#define SELECT_RATIO 3
 
 #if defined (DO_BIND_USE_SIZEOF)
 #define AF_UNIX_SOCKSIZE(S)     (sizeof S)  
