@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:45:24 1995
  * Project:     INN (innfeed)
  * File:        connection.h
- * RCSId:       $Id: connection.h,v 1.8 1996/05/19 22:17:25 brister Exp brister $
+ * RCSId:       $Id: connection.h,v 1.10 1996/12/07 01:32:17 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -80,8 +80,8 @@ Connection newConnection (Host host,
                           u_int portNum,
                           u_int respTimeout,
                           u_int closePeriod,
-                          double lowPassHigh,
-                          double lowPassLow) ;
+                          double lowPassLow,
+                          double lowPassHigh) ;
 
   /* Causes the Connection to build the network connection. */
 bool cxnConnect (Connection cxn) ;
@@ -96,9 +96,17 @@ void cxnWait (Connection cxn) ;
      it automatically reconnects to the remote. */
 void cxnFlush (Connection cxn) ;
 
-  /* The Connection issues a QUIT and then deletes itself */
+  /* The Connection sends remaining articles, then issues a QUIT and then
+     deletes itself */
 void cxnClose (Connection cxn) ;
 
+  /* The Connection drops all queueed articles, then issues a QUIT and then
+     deletes itself */
+void cxnTerminate (Connection cxn) ;
+
+  /* Blow away the connection gracelessly and immedately clean up */
+void cxnNuke (Connection cxn) ;
+
   /* Tells the Connection to take the article and handle its
      transmission. If it can't (due to queue size or whatever), then the
      function returns false. The connection assumes ownership of the
@@ -116,6 +124,10 @@ void cxnLogStats (Connection cxn, bool final) ;
      the host shovel in as many as possible. May be zero. */
 size_t cxnQueueSpace (Connection cxn) ;
 
+  /* adjust the mode no-CHECK filter values */
+void cxnSetCheckThresholds (Connection cxn,
+			    double lowFilter, double highFilter) ;
+
   /* print some debugging info. */
 void gPrintCxnInfo (FILE *fp, u_int indentAmt) ;
 void printCxnInfo (Connection cxn, FILE *fp, u_int indentAmt) ;
