@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:08:24 1995
  * Project:     INN (innfeed)
  * File:        endpoint.c
- * RCSId:       $Id: endpoint.c,v 1.14 1996/08/20 12:27:35 brister Exp $
+ * RCSId:       $Id: endpoint.c,v 1.17 1996/12/07 02:28:42 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -38,7 +38,7 @@
  */
 
 #if ! defined (lint)
-static char rcsid [] = "$Id: endpoint.c,v 1.14 1996/08/20 12:27:35 brister Exp $" ;
+static char rcsid [] = "$Id: endpoint.c,v 1.17 1996/12/07 02:28:42 brister Exp $" ;
 static void use_rcsid (const char *rId) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rId) ;
 } 
@@ -152,6 +152,9 @@ typedef struct timerqelem_s
 int debugWrites ;
 void (*gPrintInfo) (void) ;
 
+extern char *InputFile ;
+extern void addPointerFreedOnExit (char *pointerToFree) ;
+
 static int sigFlag ;
 static EndPoint mainEndPoint ;
 
@@ -239,9 +242,11 @@ EndPoint newEndPoint (int fd)
       maxEndPoints = maxFds () ;
       endPoints = CALLOC (EndPoint,maxEndPoints) ;
       ASSERT (endPoints != NULL) ;
+      addPointerFreedOnExit ((char *)endPoints) ;
 
       priorityList = CALLOC (EndPoint,maxEndPoints) ;
       ASSERT (priorityList != NULL) ;
+      addPointerFreedOnExit ((char *)priorityList) ;
 
 
       for (i = 0 ; i < maxEndPoints ; i++)
@@ -275,6 +280,9 @@ void delEndPoint (EndPoint ep)
 
   ASSERT (endPoints [ep->myFd] == ep) ;
 
+  if (mainEndPoint == ep)
+    mainEndPoint = NULL ;
+  
   if (ep->inBuffer != NULL)
     freeBufferArray (ep->inBuffer) ;
   
@@ -366,7 +374,8 @@ int prepareRead (EndPoint endp,
   endp->inClientData = clientData ;
 
   FD_SET (endp->myFd, &rdSet) ;
-  FD_SET (endp->myFd, &exSet) ;
+  if ( InputFile == NULL )
+    FD_SET (endp->myFd, &exSet) ;
 
   return 1 ;
 }
@@ -421,18 +430,15 @@ TimeoutId prepareWake (EndpTCB func, time_t timeToWake, void *clientData)
 {
   TimeoutId id ;
   time_t now = theTime() ;
-  int timeToSleep ;
 
   if (now > timeToWake)
     return 0 ;
   
-  timeToSleep = timeToWake - now ;
-  
   id = timerElemAdd (timeToWake,func,clientData) ;
 
 #if 0
   dprintf (1,"Preparing wake %d at date %ld for %d seconds\n",
-           (int) id, (long) now, timeToSleep) ;
+           (int) id, (long) now, timeToWake - now) ;
 #endif
 
   return id ;
@@ -529,7 +535,7 @@ void Run ()
                 break ;
               }
         }
-        
+      
       sval = select (highestFd + 1, &rSet, &wSet, &eSet, twait) ;
 
       timePasses () ;
@@ -553,6 +559,7 @@ void Run ()
         {
           IoStatus rval ;
           int readyCount = sval ;
+          int timesThrough = 0 ;
           
           if (sigFlag && gPrintInfo != NULL)
             {
@@ -563,11 +570,52 @@ void Run ()
           for (idx = 0 ; idx < priorityCount ; idx++)
             {
               EndPoint ep = priorityList [idx] ;
+              bool specialCheck = false ;
 
               if (readyCount > 0 && ep != NULL) 
                 {
                   int fd = ep->myFd ;
 
+                  timesThrough++ ;
+                  
+                  /* Every SELECT_RATIO times around this loop we check to
+                     see if the mainEndPoint fd is ready to read. If so we
+                     process it and do the current endpoint next time
+                     around */
+                  if (idx > 0 && mainEndPoint != NULL
+                      && ep != mainEndPoint &&
+                      (timesThrough % SELECT_RATIO) == 0)
+                    {
+                      fd_set trSet ;
+                      struct timeval tw ;
+
+                      fd = mainEndPoint->myFd ;
+                      
+                      tw.tv_sec = tw.tv_usec = 0 ;
+                      memset (&trSet,0,sizeof (trSet)) ;
+                      FD_SET (fd,&trSet) ;
+
+                      sval = select (fd + 1, &trSet,0,0,&tw) ;
+
+                      if (sval > 0)
+                        {
+                          idx-- ;
+                          ep = mainEndPoint ;
+                          FD_SET (fd,&rSet) ;
+                          FD_SET (fd,&rdSet) ;
+                          readyCount++ ;
+                          specialCheck = true ;
+                        }
+                      else if (sval < 0)
+                        {
+                          syslog (LOG_ERR,BAD_SELECT,sval) ;
+                          stopRun () ;
+                          return ;
+                        }
+                      else
+                        fd = ep->myFd ; /* back to original fd. */
+                    }
+                  
                   ep->timesSelected += (FD_ISSET (fd,&wrSet) ? 1 : 0) ;
                   ep->timesSelected += (FD_ISSET (fd,&rdSet) ? 1 : 0) ;
   
@@ -595,10 +643,14 @@ void Run ()
                       else
                         {
                           FD_SET (ep->myFd, &rdSet) ;
-                          FD_SET (ep->myFd, &exSet) ;
+                          if ( InputFile == NULL )
+                            FD_SET (ep->myFd, &exSet) ;
                         }
                     }
 
+                  if (specialCheck)
+                    continue ;
+                  
                   /* get it again as the read callback may have deleted the */
                   /* endpoint */
                   ep = priorityList [idx] ;
@@ -707,6 +759,18 @@ void setMainEndPoint (EndPoint endp)
   mainEndPoint = endp ;
 }
 
+void freeTimeoutQueue (void)
+{
+  TimerElem p, n ;
+
+  p = timeoutQueue ;
+  while (p)
+    {
+      n = p->next ;
+      FREE (p) ;
+      p = n ;
+    }
+}
 
 
 /***********************************************************************/
@@ -819,14 +883,14 @@ static IoStatus doRead (EndPoint endp)
               rval = IoDone ;
             }
         }
-      else if (i < 0 && errno != EINTR)
+      else if (i < 0 && errno != EINTR && errno != EAGAIN)
         {
           endp->myErrno = errno ;
           rval = IoFailed ;
         }
       else if (i == 0)
         rval = IoEOF ;
-      else                      /* i < 0 && errno == EINTR */
+      else                   /* i < 0 && errno == EINTR || errno == EAGAIN */
         rval = IoIncomplete ;
 
       FREE (vp) ;
@@ -919,11 +983,15 @@ static IoStatus doWrite (EndPoint endp)
           if (endp->outAmtWritten == endp->outSize)
             rval = IoDone ;
         }
-      else if (i < 0)
+      else if (i < 0 && errno != EAGAIN)
         {
           endp->myErrno = errno ;
           rval = IoFailed ;
         }
+      else if (i < 0)
+        {
+          rval = IoIncomplete ;
+        }
       else
         dprintf (1,"Wrote 0 bytes in doWrite()?\n") ;
 
@@ -952,8 +1020,10 @@ static IoStatus doExcept (EndPoint endp)
   else
     syslog (LOG_ERR,UNKNOWN_EXCEPTION,fd) ;
 
+#if 0
   sleep (5) ;
   abort () ;
+#endif
 
   /* Not reached */
   return IoFailed ;
@@ -1031,8 +1101,6 @@ static void reorderPriorityList (void)
 static TimerElem newTimerElem (TimeoutId i, time_t w, EndpTCB f, void *d)
 {
   TimerElem p = CALLOC (TimerElemStruct,1) ;
-  ASSERT (p != NULL) ;
-
 
   ASSERT (p != NULL) ;
   
@@ -1050,7 +1118,7 @@ static TimerElem newTimerElem (TimeoutId i, time_t w, EndpTCB f, void *d)
 /* add a new timeout structure to the global list. */
 static TimeoutId timerElemAdd (time_t when, EndpTCB func, void *data)
 {
-  TimerElem p = newTimerElem (++nextId,when,func,data) ;
+  TimerElem p = newTimerElem (++nextId ? nextId : ++nextId,when,func,data) ;
   TimerElem n = timeoutQueue ;
   TimerElem q = NULL ;
   
