@@ -4,7 +4,7 @@
  * Start Date:  Thu Dec 28 17:29:05 1995
  * Project:     INN (innfeed)
  * File:        host.c
- * RCSId:       $Id: host.c,v 1.18 1996/08/20 12:29:22 brister Exp $
+ * RCSId:       $Id: host.c,v 1.22 1996/12/07 01:33:06 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: host.c,v 1.18 1996/08/20 12:29:22 brister Exp $" ;
+static const char *rcsid = "$Id: host.c,v 1.22 1996/12/07 01:33:06 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -44,15 +44,27 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include <syslog.h>
 #include <sys/param.h>
 #include <stdio.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <ctype.h>
+#include <limits.h>
+#include <errno.h>
+#include <math.h>
+#include <netdb.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <limits.h> /* LONG_MAX */
 
 #include "host.h"
 #include "tape.h"
 #include "connection.h"
 #include "article.h"
+#include "buffer.h"
 #include "endpoint.h"
 #include "innlistener.h"
 #include "msgs.h"
 
+extern char *configFile ;
 
 /* the host keeps a couple lists of these */
 typedef struct proc_q_elem 
@@ -67,6 +79,8 @@ struct host_s
     InnListener listener ;      /* who created me. */
     char *peerName ;            /* name INN calls the remote */
     char *ipName ;              /* IP name/address from config file. */
+    char **ipAddrs ;		/* the ip addresses of the remote */
+    char **nextIpAddr ;		/* the next ip address to hand out */
 
     Connection *connections ;   /* NULL-terminated list of all connections */
     bool *cxnActive ;           /* true if the corresponding cxn is active */
@@ -76,6 +90,8 @@ struct host_s
     u_int sleepingCxns ;        /* number of connections currently sleeping */
     u_int initialCxns ;         /* number of connections to create at start */
     Connection blockedCxn ;     /* the first connection to get the 400 banner*/
+    double lowPassLow ;		/* value of low-pass filter off threshold */
+    double lowPassHigh ;	/* value of low-pass filter on threshold */
 
     bool wantStreaming ;        /* true if config file says to stream. */
     bool remoteStreams ;        /* true if remote supports streaming */
@@ -104,6 +120,7 @@ struct host_s
 
     bool loggedBacklog ;        /* true if we already logged the fact */
     bool notifiedChangedRemBlckd ; /* true if we logged a new response 400 */
+    bool removeOnReload ;	/* set to false when host is in config */
 
     /* these numbers get reset periodically (after a 'final' logging). */
     u_int artsOffered ;         /* # of articles we offered to remote. */
@@ -131,6 +148,7 @@ struct host_s
                                    spooling articles to tape. */
     time_t lastSpoolTime ;      /* the time the last time the Host had to
                                    revert to spooling articles to tape. */
+    time_t nextIpLookup ;	/* time of last IP name resolution */
 
     Host next ;                 /* for global list of hosts. */
 };
@@ -138,6 +156,15 @@ struct host_s
 
 
 
+static u_int defaultArticleTimeout ;
+static u_int defaultResponseTimeout ;
+static u_int defaultInitialConnections ;
+static u_int defaultMaxConnections ;
+static u_int defaultMaxChecks ;
+static bool defaultStreaming ;
+static double defaultLowFilter ;
+static double defaultHighFilter ;
+static u_short defaultPortNumber ;
 
   /*
    * Host object private methods.
@@ -153,6 +180,23 @@ static bool amClosing (Host host) ;
 static void hostLogStatus (void) ;
 static void hostPrintStatus (Host host, FILE *fp) ;
 
+static bool getHostInfo (FILE *fp,
+                         char **name,
+                         char **ipname,
+                         u_int *articleTimeout,
+                         u_int *responseTimeout,
+                         u_int *initialConnections,
+                         u_int *maxConnections,
+                         u_int *maxChecks,
+                         bool *streaming,
+                         double *lowFilter,
+                         double *highFilter,
+                         u_short *portNumber) ;
+static Host findHostByName (char *name) ;
+static bool getOptUInt (u_int *field, char *line, u_int fNum, u_int defVal) ;
+static bool getOptBool (bool *field, char *line, u_int fNum, bool defVal) ;
+static bool getOptDouble (double *field, char *line, u_int fieldNum,
+                          double defVal) ;
 
 /* article queue management functions */
 static Article remHead (ProcQElem *head, ProcQElem *tail) ;
@@ -169,7 +213,7 @@ static bool remArticle (Article article, ProcQElem *head, ProcQElem *tail) ;
 
 /* if true then when a Host logs its stats, it has all its connections
    log theirs too. */
-static bool logConnectionStats = false ;
+static bool logConnectionStats = (bool) LOG_CONNECTION_STATS ;
 
 /* The frequency in seconds with which a Host will log its stats. */
 static u_int statsPeriod = STATS_PERIOD ;
@@ -195,6 +239,167 @@ static pid_t myPid ;
 
 
 
+void configHosts (bool talkSelf)
+{
+  FILE *fp ;
+  Host nHost, h ;
+  char *name, *ipName ;
+  u_int artTout, respTout, initCxns, maxCxns, maxChecks ;
+  double lowFilter, highFilter ;
+  bool streaming ;
+  u_short portNum ;
+
+  name = ipName = NULL ;
+
+  if ( strlen (configFile) == 0 || !fileExistsP (configFile) )
+    {
+      syslog (LOG_ERR, NOSUCH_CONFIG, configFile) ;
+      dprintf (1,"No such config file: %s\n", configFile) ;
+      exit (1) ;
+    }
+
+  if ((fp = fopen (configFile,"r")) == NULL)
+    {
+      syslog (LOG_ERR,FOPEN_FAILURE, configFile) ;
+      exit (1) ;
+    }
+
+  syslog(LOG_NOTICE, "loading %s", configFile) ;
+
+  openfds++ ;
+
+  while (getHostInfo (fp, &name, &ipName, &artTout, &respTout, &initCxns,
+                      &maxCxns, &maxChecks, &streaming, &lowFilter,
+                      &highFilter,&portNum))
+    {
+      h = findHostByName (name) ;
+      if ( h != NULL )
+        {
+	  int i ;
+
+	  if (strcmp(h->ipName, ipName) != 0)
+	    {
+	      FREE (h->ipName) ;
+	      h->ipName = strdup (ipName) ;
+	      h->nextIpLookup = theTime () ;
+	    }
+
+	  h->articleTimeout = artTout ;
+	  h->responseTimeout = respTout ;
+	  h->maxChecks = maxChecks ;
+	  h->wantStreaming = streaming ;
+	  h->port = portNum ;
+	  h->removeOnReload = false ;
+ 	  h->lowPassLow = lowFilter ;   /* the off threshold */
+ 	  h->lowPassHigh = highFilter ; /* the on threshold */
+
+	  if (maxCxns != h->maxConnections)
+	    {
+	      if (maxCxns < h->maxConnections)
+		{
+		  for ( i = h->maxConnections ; i > maxCxns ; i-- )
+		    {
+		      /* XXX this is harsh, and arguably there could be a
+			 cleaner way of doing it.  the problem being addressed
+			 by doing it this way is that eventually a connection
+			 closed cleanly via cxnClose can end up ultimately
+			 calling hostCxnDead after h->maxConnections has
+			 been lowered and the relevant arrays downsized.
+			 If trashing the old, unallocated space in
+			 hostCxnDead doesn't kill the process, the
+			 ASSERT against h->maxConnections surely will.
+		       */
+		      cxnNuke (h->connections[i-1]) ;
+		    }
+		  h->maxConnections = maxCxns ;
+		}
+
+	      h->connections =
+		REALLOC (h->connections, Connection, maxCxns + 1);
+	      ASSERT (h->connections != NULL) ;
+	      h->cxnActive = REALLOC (h->cxnActive, bool, maxCxns) ;
+	      ASSERT (h->cxnActive != NULL) ;
+	      h->cxnSleeping = REALLOC (h->cxnSleeping, bool, maxCxns) ;
+	      ASSERT (h->cxnSleeping != NULL) ;
+
+	      /* if maximum was raised, establish the new connexions
+		 (but don't start using them).
+		 XXX maybe open them based on initCxns? */
+	      if (maxCxns > h->maxConnections)
+		{
+		  i = h->maxConnections ;
+		  /* need to set h->maxConnections before cxnWait() */
+		  h->maxConnections = maxCxns ;
+		  while ( i < maxCxns )
+		    {
+		      /* XXX this does essentially the same thing that happens
+			 in newHost, so they should probably be combined
+			 to one new function */
+		      h->connections [i] =
+			newConnection (h, i,
+				       h->ipName,
+				       h->articleTimeout,
+				       h->port,
+				       h->responseTimeout,
+				       CLOSE_PERIOD, lowFilter, highFilter) ;
+		      h->cxnActive [i] = false ;
+		      h->cxnSleeping [i] = false ;
+		      cxnWait (h->connections [i]) ;
+		      i++ ;
+		    }
+		}
+	    }
+
+	  for ( i = 0 ; i < maxCxns ; i++ )
+	    cxnSetCheckThresholds (h->connections[i], lowFilter, highFilter) ;
+
+	  /* XXX how to handle initCxns change? */
+        }
+      else
+        {
+	  nHost = newHost (mainListener,name,ipName,artTout,respTout,initCxns,
+			   maxCxns,maxChecks,portNum,CLOSE_PERIOD,
+			   streaming,lowFilter,highFilter) ;
+	  
+	  /* this is where we'll die if the locks are still held. */
+	  if (nHost == 0)
+	    {
+	      syslog (LOG_ERR,NO_HOST,name) ;
+	    }
+	  else 
+	    {
+	      if (initCxns == 0 && talkSelf)
+		syslog (LOG_NOTICE,BATCH_AND_NO_CXNS,name) ;
+	      
+	      dprintf (1,"Adding %s %s article (%d) response (%d) initial (%d) max con (%d) max checks (%d) portnumber (%d) streaming (%s) lowFilter (%.2f) highFilter (%.2f)\n",
+		       name, ipName, artTout, respTout, initCxns, maxCxns,
+		       maxChecks, portNum, streaming ? "true" : "false",
+		       lowFilter, highFilter) ;
+	      
+	      if ( !listenerAddPeer (mainListener,nHost) )
+		die ("failed to add a new peer\n") ;
+	    }
+	}
+    }
+  
+
+  if ( !feof (fp) )
+    {
+      syslog (LOG_ERR,PARSE_FAILURE) ;
+      exit (1) ;
+    }
+
+  fclose (fp) ;
+  openfds-- ;
+
+  for (h = gHostList; h != NULL; h = h->next)
+    if (h->removeOnReload)
+      hostClose (h) ;
+    else
+      /* prime it for the next config file read */
+      h->removeOnReload = true ;
+}
+
 /*
  * Create a new Host object. Called by the InnListener.
  */
@@ -210,8 +415,8 @@ Host newHost (InnListener listener,
               u_short portNum,
               u_int closePeriod,
               bool streaming,
-              double lowPassHigh,
-              double lowPassLow) 
+              double lowPassLow,
+              double lowPassHigh) 
 {
   u_int i ;
   Host nh ; 
@@ -225,7 +430,7 @@ Host newHost (InnListener listener,
   nh->listener = listener ;
   nh->peerName = strdup (name) ;
   nh->ipName = strdup (ipName) ;
-  
+
   nh->connections = CALLOC (Connection, maxCxns + 1) ;
   ASSERT (nh->connections != NULL) ;
 
@@ -233,12 +438,14 @@ Host newHost (InnListener listener,
   ASSERT (nh->cxnActive != NULL) ;
 
   nh->cxnSleeping = CALLOC (bool, maxCxns) ;
-  ASSERT (nh->cxnActive != NULL) ;
+  ASSERT (nh->cxnSleeping != NULL) ;
 
   nh->maxConnections = maxCxns ;
   nh->activeCxns = 0 ;
   nh->sleepingCxns = 0 ;
   nh->initialCxns = initialCxns ;
+  nh->lowPassLow = lowPassLow ;
+  nh->lowPassHigh = lowPassHigh ;
 
   nh->blockedCxn = NULL ;
   nh->maxChecks = maxCheck ;
@@ -273,6 +480,7 @@ Host newHost (InnListener listener,
   nh->loggedModeOn = false ;
   nh->loggedModeOff = false ;
   nh->notifiedChangedRemBlckd = false ;
+  nh->removeOnReload = false ;
 
   nh->artsOffered = 0 ;
   nh->artsAccepted = 0 ;
@@ -306,8 +514,8 @@ Host newHost (InnListener listener,
                                            nh->port,
                                            nh->responseTimeout,
                                            closePeriod,
-                                           lowPassHigh,
-                                           lowPassLow) ;
+                                           nh->lowPassLow,
+                                           nh->lowPassHigh) ;
       if (i < initialCxns)
         cxnConnect (nh->connections [i]) ;
       else
@@ -335,6 +543,86 @@ Host newHost (InnListener listener,
   return nh ;
 }
 
+struct in_addr *hostIpAddr (Host host)
+{
+  int i ;
+  char *p ;
+  char **newIpAddrs = NULL;
+  struct in_addr ipAddr, *returnAddr ;
+  struct hostent *hostEnt ;
+
+  /* check to see if need to look up the host name */
+  if (host->nextIpLookup <= theTime () )
+    {
+      /* see if the ipName we're given is a dotted quad */
+      if ( !inet_aton (host->ipName,&ipAddr) )
+	{
+	  if ((hostEnt = gethostbyname (host->ipName)) == NULL)
+	    syslog (LOG_ERR, HOST_RESOLV_ERROR, host->peerName, host->ipName,
+		    host_err_str ()) ;
+	  else
+	    {
+	      /* figure number of pointers that need space */
+	      for (i = 0 ; hostEnt->h_addr_list[i] ; i++)
+		;
+	      i++;		
+
+	      newIpAddrs =
+		(char **) MALLOC ( (i * sizeof(char *)) +
+				  ( (i - 1) * hostEnt->h_length) ) ;
+	      ASSERT (newIpAddrs != NULL) ;
+
+	      /* copy the addresses from gethostbyname() static space */
+	      p = (char *)&newIpAddrs [ i ] ;
+	      i = 0;
+	      for (i = 0 ; hostEnt->h_addr_list[i] ; i++)
+		{
+		  newIpAddrs[i] = p;
+		  memcpy (p, hostEnt->h_addr_list[i], hostEnt->h_length) ;
+		  p += hostEnt->h_length ;
+		}
+	      newIpAddrs[i] = NULL ;
+	    }
+	}
+      else
+	{
+	  newIpAddrs = (char **) MALLOC (2 * sizeof(char *) + sizeof(ipAddr)) ;
+	  ASSERT (newIpAddrs != NULL) ;
+	  p = (char *)&newIpAddrs [ 2 ];
+	  newIpAddrs[0] = p;
+	  memcpy (p, (char *)&ipAddr, sizeof(ipAddr)) ;
+	  newIpAddrs[1] = NULL;
+	}
+
+      if (newIpAddrs)
+	{
+	  if (host->ipAddrs)
+	    FREE (host->ipAddrs) ;
+	  host->ipAddrs = newIpAddrs ;
+	  host->nextIpAddr = host->ipAddrs ;
+	  /* XXX should be configurable */
+	  host->nextIpLookup = theTime () + 86400 ;
+	}
+      else
+	{
+	  /* failed to setup new addresses */
+	  /* XXX should be configurable */
+	  host->nextIpLookup = theTime () + 900 ;
+	}
+    }
+
+  if (host->ipAddrs)
+    {
+      returnAddr = (struct in_addr *)(host->nextIpAddr[0]) ;
+      if (*(++host->nextIpAddr) == NULL)
+	host->nextIpAddr = host->ipAddrs ;
+    }
+  else
+    returnAddr = NULL ;
+
+  return returnAddr ;
+}
+
 
 void gPrintHostInfo (FILE *fp, u_int indentAmt)
 {
@@ -514,7 +802,7 @@ void hostClose (Host host)
   host->connectTime = 0 ;
 
   for (i = 0 ; i < host->maxConnections ; i++)
-    cxnClose (host->connections [i]) ;
+    cxnTerminate (host->connections [i]) ;
 }
 
 
@@ -717,6 +1005,7 @@ void hostCxnDead (Host host, Connection cxn)
               {
                 clearTimer (host->statsId) ;
                 hostLogStats (host,true) ;
+                host->connectTime = 0 ;
               }
           }
 
@@ -800,9 +1089,9 @@ void hostCxnWaiting (Host host, Connection cxn)
 /*
  * Called by the Connection when it is about to delete itself.
  */
-void hostCxnGone (Host host, Connection cxn)
+bool hostCxnGone (Host host, Connection cxn)
 {
-  u_int i ;
+  u_int i;
   bool oneThere = false ;
 
   /* forget about the Connection and see if we are still holding any live
@@ -819,10 +1108,12 @@ void hostCxnGone (Host host, Connection cxn)
     else if (host->connections [i] != NULL)
       oneThere = true ;
 
+  /* remove the host if it has no connexions */
   if ( !oneThere )
     {
       time_t now = theTime() ;
-      
+      u_int hostsLeft ;
+
       if (host->firstConnectTime > 0)
         syslog (LOG_NOTICE,REALLY_FINAL_STATS,
                 host->peerName,
@@ -831,9 +1122,15 @@ void hostCxnGone (Host host, Connection cxn)
                 host->gArtsNotWanted, host->gArtsRejected,
                 host->gArtsMissing) ;
 
-      listenerHostGone (host->listener, host) ;
+      hostsLeft = listenerHostGone (host->listener, host) ;
       delHost (host) ;
+
+      /* return true if that was the last host */
+      return hostsLeft == 0 ;
     }
+
+  /* return false because there is still at least one host (this one) */
+  return false ;
 }
 
 
@@ -1178,6 +1475,225 @@ void hostLogNoCheckMode (Host host, bool on)
 
 
 
+  /* parse lines of the form used in the config file:
+
+     name:fqdn:article-timeout:response-timeout:initial-connections:
+     max-connections:max-q-size:streaming:low-filter:high-filter:portnum
+
+     XXX i'm not handling continuation lines yet.
+     */
+
+
+static bool getHostInfo (FILE *fp,
+                         char **name,
+                         char **ipName,
+                         u_int *articleTimeout,
+                         u_int *responseTimeout,
+                         u_int *initialConnections,
+                         u_int *maxConnections,
+                         u_int *maxChecks,
+                         bool *streaming,
+                         double *lowFilter,
+                         double *highFilter,
+                         u_short *portNumber) 
+{
+  static bool seenDefault ;
+  static char line [1024] ;
+  char *p, *l, copyLine [1024] ;
+  char *field1 = 0;
+  char *field2 = 0;
+  u_int field3, field4, field5, field6, field7 ;
+  bool field8 ;
+  double field9, field10 ;
+  u_int field11 ;
+  bool gotSomething = false ;
+
+  while (fgets (line, sizeof (line), fp) != NULL)
+    {
+      gotSomething = false ;
+      dprintf (1,"Doing config line: %s",line) ;
+      
+      strcpy (copyLine,line) ;  /* copyLine is used for syslog only */
+      
+      for (l = line ; *l != '\0' && isspace (*l) ; l++)
+          /* nada */ ;
+      
+      if ((p = strchr (l,'#')) == NULL)
+        p = l + strlen (l) - 1 ;
+      else
+        p-- ;
+
+      while (p > l && isspace (*p))
+        p-- ;
+
+      *++p = '\0' ;
+
+      if (strlen (l) == 0) 
+        continue ;
+
+        /* field 1 the system name */
+      if (((field1 = mystrtok (l,":")) == NULL) || (strlen (field1) == 0))
+        {
+          dprintf (1,"Bad system name in config file line: %s\n", copyLine) ;
+          return false ;
+        }
+      trim_ws (field1) ;
+
+        /* field 2 the fqdn */
+      if ((field2 = mystrtok (NULL,":")) == NULL)
+        {
+          dprintf (1,"Bad system IP name in config file line: %s\n", copyLine) ;
+          return false ;
+        }
+      trim_ws (field2) ;
+
+      if ((strcmp (field1, "default") != 0) && (strlen (field2) == 0))
+        {
+          dprintf (1,"f2 - 1\n") ;
+          return false ;
+        }
+      else if ((strcmp (field1, "default") == 0) && (strlen (field2) != 0))
+        {
+          dprintf (1,"f2 - 2\n") ;
+          return false ;
+        }
+
+
+        /* field 3 the article timeout */
+      if ( !getOptUInt (&field3,copyLine,3,defaultArticleTimeout) )
+        return false ;
+
+        /* field 4 the response timeout */
+      if ( !getOptUInt (&field4,copyLine,4,defaultResponseTimeout) )
+        return false ;
+
+        /* field 5 the number of initial connections to create */
+      if ( !getOptUInt (&field5,copyLine,5,defaultInitialConnections) )
+        return false ;
+      
+        /* field 6 the max # of connections */
+      if ( !getOptUInt (&field6,copyLine,6,defaultMaxConnections) )
+        return false ;
+
+        /* field 7 the max number of checks */
+      if ( !getOptUInt (&field7,copyLine,7,defaultMaxChecks) )
+        return false ;
+
+      /* field 8 the streaming flag */
+      if ( !getOptBool (&field8,copyLine,8,defaultStreaming) )
+        return false ;
+
+      /* field 9 the low-pass filter low value. */
+      if ( !getOptDouble (&field9,copyLine,9,defaultLowFilter) )
+        return false ;
+
+      /* field 10 the low-pass filter high value */
+      if ( !getOptDouble (&field10,copyLine,10,defaultHighFilter) )
+        return false ;
+
+      /* field 11 the port number */
+      if ( !getOptUInt (&field11,copyLine,11,defaultPortNumber) )
+        return false ;
+
+      if (field9 > field10)
+        {
+          syslog (LOG_ERR,LOW_GREATER,field1) ;
+          exit (1) ;
+        }
+      
+      if (strcmp (field1,"default") != 0)
+        {
+          gotSomething = true ;
+          break ;
+        }
+
+      seenDefault = true ;
+      
+      defaultArticleTimeout = field3 ;
+      defaultResponseTimeout = field4 ;
+      defaultInitialConnections = field5 ;
+      defaultMaxConnections = field6 ;
+      defaultMaxChecks = field7 ;
+      defaultStreaming = field8 ;
+      defaultLowFilter = field9 ;
+      defaultHighFilter = field10 ;
+      defaultPortNumber = field11 ;
+
+      dprintf (1,"defaults are set %d %d %d %d %d %s %.2f %.2f %d\n",
+               defaultArticleTimeout,
+               defaultResponseTimeout,
+               defaultInitialConnections,
+               defaultMaxConnections,
+               defaultMaxChecks,
+               defaultStreaming ? "true" : "false",
+               defaultLowFilter,
+               defaultHighFilter,
+               defaultPortNumber
+               ) ;
+      
+    }
+
+  if ( !gotSomething )
+    return false ;
+
+  if ( !seenDefault )
+    {
+      syslog (LOG_ERR,NO_DEFAULT) ;
+      return false ;
+    }
+
+  *name = field1 ;
+  *ipName = field2 ;
+  *articleTimeout = field3 ;
+  *responseTimeout = field4 ;
+  *initialConnections = field5 ;
+  *maxConnections = field6 ;
+  *maxChecks = field7 ;
+  *streaming = field8 ;
+  *lowFilter = field9 ;
+  *highFilter = field10 ;
+  *portNumber = field11 ;
+
+  return true ;
+}
+
+
+
+void getHostDefaults (u_int *articleTout,
+                      u_int *respTout,
+                      u_int *initialCxns,
+                      u_int *maxCxns,
+                      u_int *maxChecks,
+                      bool *streaming,
+                      double *lowFilter,
+                      double *highFilter,
+                      u_short *portNum)
+{
+  ASSERT (defaultMaxConnections > 0) ;
+  
+  ASSERT (articleTout != NULL) ;
+  ASSERT (respTout != NULL) ;
+  ASSERT (initialCxns != NULL) ;
+  ASSERT (maxCxns != NULL) ;
+  ASSERT (maxChecks != NULL) ;
+  ASSERT (portNum != NULL) ;
+  ASSERT (streaming != NULL) ;
+  ASSERT (lowFilter != NULL) ;
+  ASSERT (highFilter != NULL) ;
+  
+  
+  *articleTout = defaultArticleTimeout ;
+  *respTout = defaultResponseTimeout ;
+  *initialCxns = defaultInitialConnections ;
+  *maxCxns = defaultMaxConnections ;
+  *maxChecks = defaultMaxChecks ;
+  *streaming = defaultStreaming ;
+  *lowFilter = defaultLowFilter ;
+  *highFilter = defaultHighFilter ;
+  *portNum = defaultPortNumber ;
+}
+
+
 /*
  * fully delete and clean up the Host object.
  */
@@ -1201,16 +1717,29 @@ void delHost (Host host)
   
   FREE (host->connections) ;
   FREE (host->cxnActive) ;
+  FREE (host->cxnSleeping) ;
   FREE (host->peerName) ;
   FREE (host->ipName) ;
 
+  if (host->ipAddrs)
+    FREE (host->ipAddrs) ;
+
   FREE (host) ;
   gHostCount-- ;
 }
 
 
+
+static Host findHostByName (char *name) 
+{
+  Host h;
 
+  for (h = gHostList; h != NULL; h = h->next)
+    if ( strcmp(h->peerName, name) == 0 )
+      return h;
 
+  return NULL;
+}
 
 
 
@@ -1374,10 +1903,16 @@ static void hostLogStatus (void)
   FILE *fp = NULL ;
   Host h ;
   bool anyToLog = false ;
+  u_int peerNum = 0, actConn = 0, slpConn = 0 ;
 
   for (h = gHostList ; h != NULL ; h = h->next)
     if (h->myTape != NULL)   /* the host deletes its tape when it's closing */
-      anyToLog = true ;
+      {
+        anyToLog = true ;
+        peerNum++ ;
+        actConn += h->activeCxns ;
+        slpConn += h->sleepingCxns ;
+      }
 
   if (!anyToLog)
     return ;
@@ -1398,18 +1933,59 @@ static void hostLogStatus (void)
 
       now = time (NULL) ;
       strcpy (timeString,ctime (&now)) ;
-      
-      fprintf (fp,"%s\npid %d started %s\nUpdated: %s\n",
+
+#if GEN_HTML
+      fprintf (fp,"<HTML><META HTTP-EQUIV=\"Refresh\" CONTENT=\"20;\"><PRE>\n\n") ;
+#endif
+
+      fprintf (fp,"%s\npid %d started %s\nUpdated: %s",
                versionInfo,(int) myPid,startTime,timeString) ;
+      fprintf (fp," (peers: %d active: %d sleeping: %d)\n\n",
+               peerNum, actConn, slpConn) ;
+               
+      fprintf (fp,"Default configuration parameters:\n") ;
+
+      fprintf (fp,"    article timeout: %-5d     initial connections: %d\n",
+	       defaultArticleTimeout, defaultInitialConnections) ;
+
+      fprintf (fp,"   response timeout: %-5d         max connections: %d\n",
+	       defaultResponseTimeout, defaultMaxConnections) ;
+
+      fprintf (fp,"         max checks: %-5d               streaming: %s\n",
+	       defaultMaxChecks, (defaultStreaming ? "true" : "false")) ;
+
+      fprintf (fp,"       on threshold: %-2.1f%%                port num: %d\n",
+	       (defaultHighFilter * 10), defaultPortNumber) ;
+
+      fprintf (fp,"      off threshold: %-2.1f%%\n\n",
+	       (defaultLowFilter * 10)) ;
+
       for (h = gHostList ; h != NULL ; h = h->next)
         hostPrintStatus (h,fp) ;
 
+#if GEN_HTML
+      fprintf (fp,"</PRE></HTML>\n") ;
+#endif
+      
       fclose (fp) ;
     }
 }
 
 
-
+/*
+ * This prints status information for each host.  An example of the
+ * format of the output is:
+ *
+ * sitename
+ *   seconds: 351       art. timeout: 400          ip name: foo.bar
+ *   offered: 1194     resp. timeout: 240             port: 119
+ *  accepted: 178     want streaming: yes      active cxns: 6
+ *   refused: 948       is streaming: yes    sleeping cxns: 0
+ *  rejected: 31          max checks: 25      initial cxns: 5
+ *   missing: 0         on threshold: 95.0%       max cxns: 6
+ *  deferred: 0        off threshold: 95.0%
+ *
+ */
 static void hostPrintStatus (Host host, FILE *fp)
 {
   time_t now = theTime() ;
@@ -1417,24 +1993,38 @@ static void hostPrintStatus (Host host, FILE *fp)
   ASSERT (host != NULL) ;
   ASSERT (fp != NULL) ;
 
-  fprintf (fp,"%s ip-name: %s type: %s\n",
-           host->peerName, host->ipName,
-           (host->remoteStreams ? "streaming" : "non-streaming")) ;
+  fprintf (fp,"%s\n",
+	   host->peerName);
            
-  fprintf (fp,"    article-timeout: %d response-timeout: %d\n",
-           host->articleTimeout,host->responseTimeout) ;
+  fprintf (fp, "   seconds: %-7ld   art. timeout: %-5d        ip name: %s\n",
+	   host->firstConnectTime > 0 ? (long) (now - host->firstConnectTime) : 0,
+	   host->articleTimeout, host->ipName) ;
            
-  fprintf (fp,"    active-connections: %d sleeping-connections: %d\n",
-           host->activeCxns,host->sleepingCxns) ;
+  fprintf (fp, "   offered: %-7ld  resp. timeout: %-5d           port: %d\n",
+	   (long) host->gArtsOffered, host->responseTimeout, host->port);
 
-  if (host->firstConnectTime > 0)
-    fprintf (fp,"    seconds %ld offered %ld accepted %ld refused %ld rejected %ld missing %ld\n\n",
-             (long) (now - host->firstConnectTime),
-             (long) host->gArtsOffered, (long) host->gArtsAccepted,
-             (long) host->gArtsNotWanted, (long) host->gArtsRejected,
-             (long) host->gArtsMissing) ;
-  else
-    fprintf (fp,"\n") ;
+  fprintf (fp, "  accepted: %-7ld want streaming: %s      active cxns: %d\n",
+	   (long) host->gArtsAccepted, 
+           (host->wantStreaming ? "yes" : "no "),
+	   host->activeCxns) ;
+
+  fprintf (fp, "   refused: %-7ld   is streaming: %s    sleeping cxns: %d\n",
+	   (long) host->gArtsNotWanted,
+           (host->remoteStreams ? "yes" : "no "),
+	   host->sleepingCxns) ;
+
+  fprintf (fp, "  rejected: %-7ld     max checks: %-5d   initial cxns: %d\n",
+	   (long) host->gArtsRejected, host->maxChecks,
+	   host->initialCxns) ;
+
+  fprintf (fp, "   missing: %-7ld   on threshold: %-2.1f%%       max cxns: %d\n",
+	   (long) host->gArtsMissing, (host->lowPassHigh * 10),
+	   host->maxConnections) ;
+
+  fprintf (fp, "  deferred: %-7ld  off threshold: %-2.1f%%\n",
+	   (long) host->gArtsDeferred, (host->lowPassLow * 10)) ;
+
+  fprintf (fp, "\n");
 }
 
 
@@ -1631,3 +2221,107 @@ static Article remHead (ProcQElem *head, ProcQElem *tail)
   return art ;
 }
 
+static bool getOptDouble (double *field, char *line, u_int fieldNum,
+                          double defVal)
+{
+  char *tok, *ptr ;
+
+  if ((tok = mystrtok (NULL,":")) == NULL)
+    *field = defVal ;
+  else if (strlen (tok) > 0)
+    {
+      errno = 0 ;
+
+      /* yeah, I know I could collapse these all into one giant if. */
+      if (((*field = strtod (tok, &ptr)) == 0) && (ptr == tok))
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+      else if (*ptr != '\0')
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+      else if ((*field == -(HUGE_VAL) || *field == HUGE_VAL) &&
+               errno == ERANGE)
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+      else if (*field == 0 && errno == ERANGE)
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+    }
+  else
+    *field = defVal ;
+
+  return true ;
+}
+
+static bool getOptBool (bool *field, char *line, u_int fieldNum, bool defVal)
+{
+  char *tok, *ptr, *p ;
+  u_int intVal ;
+
+  if ((tok = mystrtok (NULL,":")) == NULL)
+    *field = defVal ;
+  else if (strlen (tok) > 0)
+    {
+      intVal = strtol (tok, &ptr, 10) ;
+      if (ptr == tok)
+        {
+          for (p = tok ; *p && isspace (*p) ; p++)
+              /* nada */ ;
+          if ( *p == 't' || *p == 'T' )
+            *field = true ;
+          else if ( *p == 'f' || *p == 'F' )
+            *field = false ;
+          else
+            {
+              dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+              return false ;
+            }
+        }
+      else if (ptr != '\0')
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+      else if (intVal == 0)
+        *field = false ;
+      else if (intVal == 1)
+        *field = true ;
+      else
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+    }
+  else
+    *field = defVal ;
+
+  return true ;
+}
+
+static bool getOptUInt (u_int *field, char *line, u_int fieldNum, u_int defVal)
+{
+  char *tok, *ptr ;
+
+  if ((tok = mystrtok (NULL,":")) == NULL)
+    *field = defVal ;
+  else if (strlen (tok) > 0)
+    {
+      if (((*field = strtol (tok, &ptr, 10)) == LONG_MAX) || (*ptr != '\0'))
+        {
+          dprintf (1,"Bad field %d in line: %s\n", fieldNum, line) ;
+          return false ;
+        }
+    }
+  else
+    *field = defVal ;
+
+  return true ;
+}
