@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:44:20 1995
  * Project:     INN (innfeed)
  * File:        host.h
- * RCSId:       $Id: host.h,v 1.10 1996/08/20 12:30:44 brister Exp $
+ * RCSId:       $Id: host.h,v 1.11 1996/11/20 18:02:44 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -93,6 +93,8 @@ Host newHost (InnListener listener,
               double lowPassHigh) ;
 
 
+void configHosts (bool talkSelf) ;
+
 /* print some debugging info. */
 void gPrintHostInfo (FILE *fp, u_int indentAmt) ;
 void printHostInfo (Host host, FILE *fp, u_int indentAmt) ;
@@ -112,7 +114,8 @@ void hostFlush (Host host) ;
    information for later attempts. */
 void hostSendArticle (Host host, Article article) ;
 
-
+/* return an IP address for the host */
+struct in_addr *hostIpAddr (Host host) ;
 
 /*
  * Functions used by the Connection to indicate Connection state.
@@ -131,7 +134,7 @@ void hostRemoteStreams (Host host, Connection cxn, bool doesStream) ;
 void hostCxnDead (Host host, Connection cxn) ;
 
 /* Called when the Connection deletes itself */
-void hostCxnGone (Host host, Connection cxn) ;
+bool hostCxnGone (Host host, Connection cxn) ;
 
 /* Called when the Connection goes to sleep. */
 void hostCxnSleeping (Host host, Connection cxn) ;
