@@ -4,7 +4,7 @@
  * Start Date:  Thu Dec 28 13:15:04 1995
  * Project:     INN (innfeed)
  * File:        innlistener.c
- * RCSId:       $Id: innlistener.c,v 1.18 1996/08/20 14:07:47 brister Exp $
+ * RCSId:       $Id: innlistener.c,v 1.20 1996/11/20 18:02:44 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: innlistener.c,v 1.18 1996/08/20 14:07:47 brister Exp $" ;
+static const char *rcsid = "$Id: innlistener.c,v 1.20 1996/11/20 18:02:44 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -49,6 +49,7 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include <errno.h>
 #include <stdio.h>
 #include <time.h>
+#include <fcntl.h>
 
 #include "innlistener.h"
 #include "endpoint.h"
@@ -57,8 +58,8 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include "article.h"
 #include "msgs.h"
 
-#define LISTENER_INPUT_BUFFER 2048 /* size of the input buffer */
-
+#define LISTENER_INPUT_BUFFER (1024 * 8) /* byte size of the input buffer */
+#define EOF_SLEEP_TIME 1	/* seconds to sleep when EOF on InputFile */
 
 struct innlistener_s 
 {
@@ -69,6 +70,7 @@ struct innlistener_s
     Buffer inputBuffer ;
     bool dummyListener ;
     bool dynamicPeers ;
+    TimeoutId inputEOFSleepId ;
 
     InnListener next ;
 };
@@ -79,14 +81,19 @@ static InnListener listenerList = NULL ;
 InnListener mainListener ;
 
 
+extern char *InputFile ;
+extern bool RollInputFile ;
+extern void addPointerFreedOnExit (char *pointerToFree) ;
 
 static void giveArticleToPeer (InnListener lis,
                                Article article, const char *peerName) ;
 static void newArticleCommand (EndPoint ep, IoStatus i,
                                Buffer *buffs, void *data) ;
+static void wakeUp (TimeoutId id, void *data) ;
 static void logBadPeer (const char *peer) ;
 static bool isBadPeer (const char *peer) ;
 static void rememberBadPeer (const char *peer) ;
+static void writeCheckPoint (int offsetAdjust) ;
 
 
 
@@ -107,6 +114,11 @@ InnListener newListener (EndPoint endp, bool isDummy, bool dynamicPeers)
   l->dummyListener = isDummy ;
   l->dynamicPeers = dynamicPeers ;
 
+  addPointerFreedOnExit ((char *)bufferBase(l->inputBuffer)) ;
+  addPointerFreedOnExit ((char *)l->inputBuffer) ;
+  addPointerFreedOnExit ((char *)l->myHosts) ;
+  addPointerFreedOnExit ((char *)l) ;
+
   readArray = makeBufferArray (bufferTakeRef (l->inputBuffer), NULL) ;
   prepareRead (endp,readArray,newArticleCommand,l,1) ;
 
@@ -183,7 +195,12 @@ void shutDown (InnListener l)
   dprintf (1,"Shutting down the listener\n") ;
   
   if (l->myep != NULL)
-    delEndPoint (l->myep) ;
+    {
+      if (l->inputEOFSleepId != 0)
+	removeTimeout (l->inputEOFSleepId) ;
+      l->inputEOFSleepId = 0 ;
+      delEndPoint (l->myep) ;
+    }
   l->myep = NULL ;
   
   for (i = 0 ; i < l->hostLen ; i++)
@@ -218,11 +235,11 @@ bool listenerIsDummy (InnListener listener)
   return listener->dummyListener ;
 }
 
-  /* Called by the Host when it (the Host) is about to delete itself. */
-void listenerHostGone (InnListener listener, Host host)
+/* Called by the Host when it (the Host) is about to delete itself. */
+u_int listenerHostGone (InnListener listener, Host host)
 {
   u_int i ;
-  bool someThere = false ;
+  u_int someThere = 0 ;
 
   dprintf (1,"Host is gone: %s\n", hostPeerName (host)) ;
   
@@ -230,19 +247,9 @@ void listenerHostGone (InnListener listener, Host host)
     if (listener->myHosts [i] == host)
       listener->myHosts [i] = NULL ;
     else if (listener->myHosts [i] != NULL)
-      someThere = true ;
+      someThere++ ;
 
-  if (!someThere)
-    {
-      time_t now = theTime () ;
-      char dateString [30] ;
-
-      strcpy (dateString,ctime (&now)) ;
-      dateString [24] = '\0' ;
-      
-      syslog (LOG_NOTICE,STOPPING_PROGRAM,dateString) ;
-      exit (0) ;
-    }
+  return someThere ;
 }
 
 
@@ -265,6 +272,40 @@ void listenerHostIsIdle (InnListener listener, Host host)
 }
 
 
+void openInputFile ()
+{
+  int fd, i ;
+  off_t offset ;
+  char buf [32], *p ;
+
+  ASSERT (InputFile && *InputFile) ;
+
+  fd = open(InputFile, O_RDWR) ;
+  if (fd < 0)
+    die ("open %s: %s\n", InputFile, strerror(errno)) ;
+
+  if ( dup2(fd, 0) < 0 )
+    die ("dup2 %d 0: %s\n", fd, strerror(errno)) ;
+
+  (void) close (fd);
+
+  i = read(0, buf, sizeof (buf)) ;
+  if (i < 0)
+    die ("read %s: %s\n", InputFile, strerror(errno)) ;
+  else if (i > 0)
+    {
+      p = buf;
+      buf [ sizeof(buf) - 1 ] = '\0';
+      offset = (off_t) strtol (p, &p, 10) ;
+      if (offset > 0 && *p == '\n')
+	lseek (0, offset, SEEK_SET) ;
+      else
+	lseek (0, 0, SEEK_SET) ;
+    }
+  syslog(LOG_NOTICE, "ME opened %s", InputFile);
+}
+
+
 /**********************************************************************/
 /**                     STATIC PRIVATE FUNCTIONS                     **/
 /**********************************************************************/
@@ -283,25 +324,43 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
   char *bbase = bufferBase (buffs [0]) ;
   size_t blen = bufferDataSize (buffs [0]) ;
   Buffer *readArray ;
+  static int checkPointCounter ;
 
   ASSERT (ep = lis->myep) ;
 
   bufferAddNullByte  (buffs [0]) ;
   
-  if (i == IoEOF || i == IoFailed)
+  if (i == IoEOF)
     {
-      if (i == IoEOF)
+      if ( lis == mainListener && InputFile != NULL )
+	{
+	  if ( RollInputFile )
+	    {
+	      syslog(LOG_NOTICE, "ME reached EOF in %s", InputFile);
+	      openInputFile () ;
+	      RollInputFile = false ;
+	      readArray = makeBufferArray (bufferTakeRef (buffs [0]),NULL) ;
+	      prepareRead (ep, readArray, newArticleCommand, data, 1) ;
+	    }
+	  else
+	    {
+	      lis->inputEOFSleepId =
+		prepareSleep (wakeUp, EOF_SLEEP_TIME, data) ;
+
+	    }
+	}
+      else 
         {
           dprintf (1,"Got EOF on listener\n") ;
           syslog (LOG_NOTICE,INN_GONE) ;
+	  shutDown (lis) ;
         }
-      else
-        {
-          errno = endPointErrno (ep) ;
-          syslog (LOG_ERR,INN_IO_ERROR) ;
-          dprintf (1,"Got IO Error on listener\n") ;
-        }
-
+    }
+  else if (i == IoFailed)
+    {
+      errno = endPointErrno (ep) ;
+      syslog (LOG_ERR,INN_IO_ERROR) ;
+      dprintf (1,"Got IO Error on listener\n") ;
       shutDown (lis) ;
     }
   else if (strchr (bbase, '\n') == NULL) /* partial read */
@@ -379,8 +438,17 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
           delArticle (article) ;
           
           cmd = next ;
-        }
 
+	  /* write a checkpoint marker if we've done another large chunk */
+	  if (InputFile && *InputFile && ++checkPointCounter == 1000)
+	    {
+	      /* adjust the seek pointer value by the current location
+		 within the input buffer */
+	      writeCheckPoint (blen - (cmd - bbase)) ;
+	      checkPointCounter = 0 ;
+	    }
+
+        }
 
       if (*cmd != '\0')         /* partial command left in buffer */
         {
@@ -435,6 +503,19 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
   freeBufferArray (buffs) ;
 }
 
+/* EndPoint callback function for when the sleep due to 
+   having reached EOF on InputFile is done. */
+static void wakeUp (TimeoutId id, void *data)
+{
+  InnListener lis = (InnListener) data ;
+  Buffer *readArray ;
+
+  ASSERT (id = lis->inputEOFSleepId) ;
+
+  lis->inputEOFSleepId = 0 ;
+  readArray = makeBufferArray (bufferTakeRef (lis->inputBuffer), NULL) ;
+  prepareRead (lis->myep,readArray,newArticleCommand,lis,1) ;
+}
 
 /*
  * Log the name of the bad peer. One time only.
@@ -445,7 +526,7 @@ static u_int peerIdx ;
 
 static void rememberBadPeer (const char *peer)
 {
-  u_int i ;
+  u_int i = 0;
 
   if (isBadPeer (peer))         /* check if we already logged it. */
     return ;
@@ -553,3 +634,40 @@ static void giveArticleToPeer (InnListener lis,
     }
 }
 
+
+static void writeCheckPoint (int offsetAdjust)
+{
+  char offsetString[16], *writePointer ;
+  off_t offset ;
+  int writeBytes, writeReturn ;
+	      
+  offset = lseek (0, 0L, SEEK_CUR) ;
+  if (offset < 0)
+    syslog (LOG_ERR, "ME tell(0): %m") ;
+  else
+    {
+      (void) sprintf (offsetString, "%ld\n",
+		      (long)(offset - offsetAdjust) ) ;
+      if ( lseek (0, 0L, SEEK_SET) != 0 )
+	syslog (LOG_ERR, "ME seek(0, 0, 0): %m") ;
+      else
+	{
+	  writeBytes = strlen (offsetString) ;
+	  writePointer = offsetString ;
+	  do
+	    { 
+	      writeReturn = write (0, writePointer, writeBytes) ;
+	      if (writeReturn < 0)
+		{
+		  syslog (LOG_ERR,"ME write input checkpoint: %m") ;
+		  break ;
+		}
+	      writePointer += writeReturn ;
+	      writeBytes -= writeReturn ;
+	    } while (writeBytes) ;
+	  if ( lseek (0, offset, SEEK_SET) != offset )
+	    die ("ME seek(0, %ld, SEEK_SET): %s\n", (long)offset,
+		 strerror(errno) ) ;
+	}
+    }
+}
