@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:25:26 1995
  * Project:     INN (innfeed)
  * File:        innlistener.h
- * RCSId:       $Id: innlistener.h,v 1.6 1996/04/13 02:51:33 brister Exp brister $
+ * RCSId:       $Id: innlistener.h,v 1.7 1996/11/20 18:02:44 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -57,7 +57,7 @@ void gPrintListenerInfo (FILE *fp, u_int indentAmt) ;
 void printListenerInfo (InnListener listener, FILE *fp, u_int indentAmt) ;
 
   /* Called by the Host when it is about to delete itself */
-void listenerHostGone (InnListener listener, Host host) ;
+u_int listenerHostGone (InnListener listener, Host host) ;
 
     /* Called to hook up the given Host to the Listener. */
 bool listenerAddPeer (InnListener listener, Host hostObj) ;
