@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:44:47 1995
  * Project:     INN (innfeed)
  * File:        misc.h
- * RCSId:       $Id: misc.h,v 1.10 1996/08/20 13:40:12 brister Exp $
+ * RCSId:       $Id: misc.h,v 1.11 1996/11/28 19:14:30 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -154,7 +154,7 @@ void getHostDefaults (u_int *articleTout, u_int *respTout, u_int *initialCxns,
 
 const char *boolToString (bool val) ;
 
-#define ASSERT(x) do{if (!(x))die("%s in file %s line %d\n",#x,__FILE__,__LINE__);}while(0)
+#define ASSERT(x) do{if (!(x))die("%s in file %s line %d",#x,__FILE__,__LINE__);}while(0)
 
 #define INDENT_INCR 5
 #define INDENT_BUFFER_SIZE 80
