@@ -4,7 +4,7 @@
  * Start Date:  Sun, 28 Jan 1996 18:37:49 +1100
  * Project:     INN -- innfeed
  * File:        msgs.h
- * RCSId:       $Id: msgs.h,v 1.16 1996/08/20 13:40:39 brister Exp $
+ * RCSId:       $Id: msgs.h,v 1.19 1996/12/07 01:39:10 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -44,7 +44,6 @@
 
 #define CXN_BUFFER_EXPAND_ERROR "%s:%d cxnsleep can't expand input buffer"
 #define FCNTL_ERROR             "%s:%d cxnsleep can't set socket non-blocking: %m"
-#define HOST_RESOLV_ERROR       "%s:%d cxnsleep can't resolve hostname: %s : %s"
 #define IHAVE_WRITE_FAILED      "%s:%d cxnsleep can't write IHAVE body : %m"
 #define MODE_STREAM_FAILED	"%s:%d cxnsleep can't write MODE STREAM : %m"
 #define QUIT_WRITE_FAILED       "%s:%d cxnsleep can't write QUIT : %m"
@@ -52,6 +51,7 @@
 #define RESPONSE_READ_FAILED    "%s:%d cxnsleep can't read response : %m"
 #define BANNER_READ_FAILED      "%s:%d cxnsleep can't read banner : %m"
 #define SOCKET_CREATE_ERROR     "%s:%d cxnsleep can't create socket : %m"
+#define MODE_WRITE_PENDING	"%s:%s cxnsleep mode stream command still pending"
 #define NOMSGID                 "%s:%d cxnsleep message-id missing in reponse code %d: %s"
 #define INVALID_MSGID           "%s:%d cxnsleep message-id invalid message-id in reponse code %d : %s"
 #define UNKNOWN_RESPONSE        "%s:%d cxnsleep response unknown : %d : %s"
@@ -110,6 +110,7 @@
 #define REMOTE_STREAMING_OFF	"%s remote MODE STREAM disabled"
 #define REMOTE_NO_STREAMING     "%s remote MODE STREAM failed"
 #define STREAMING_CHANGE        "%s remote MODE STREAM change"
+#define HOST_RESOLV_ERROR       "%s can't resolve hostname: %s : %s"
 
 
 /* messages used in innlistener.c */
@@ -171,8 +172,10 @@
 #define NO_SNAPSHOT		"ME config ignored snapshot file: %s : %m"
 #define BATCH_AND_NO_CXNS	"%s config ignored batch mode with initial connection count of 0"
 #define LOW_GREATER		"%s config low-pass lower limit greater than upper limit"
-
-
+#define SHUTDOWN_SIGNAL		"ME received shutdown signal"
+#define CONFIG_RELOAD		"ME reloading config file %s"
+#define INCR_LOGLEVEL		"ME increasing logging level to %d"
+#define DECR_LOGLEVEL		"ME decreasing logging level to %d"
 /* misc.c */
 
 #define NO_OPEN_LOCK            "ME lock file open: %s : %m"
