@@ -4,7 +4,7 @@
  * Start Date:  Sun Dec 31 18:52:04 1995
  * Project:     INN (innfeed)
  * File:        tape.c
- * RCSId:       $Id: tape.c,v 1.14 1996/08/20 14:06:35 brister Exp $
+ * RCSId:       $Id: tape.c,v 1.15 1996/11/20 18:02:44 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -40,7 +40,7 @@
 
  
 #if ! defined (lint)
-static const char *rcsid = "$Id: tape.c,v 1.14 1996/08/20 14:06:35 brister Exp $" ;
+static const char *rcsid = "$Id: tape.c,v 1.15 1996/11/20 18:02:44 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -153,6 +153,7 @@ struct tape_s
     bool noRotate ;     
 };
 
+void addPointerFreedOnExit (char *pointerToFree) ;
 
 
 static void checkpointTape (Tape tape) ;
@@ -221,6 +222,7 @@ Tape newTape (const char *peerName, bool dontRotate)
   nt->tail = NULL ;
   
   nt->tellpos = 0 ;
+  nt->qLength = 0 ;
 
   nt->outputHasStuff = false ;
   nt->outputFileExists = false ; /* it won't */
@@ -357,6 +359,7 @@ void delTape (Tape tape)
   freeCharP (tape->outputFilename) ;
   freeCharP (tape->lockFilename) ;
   freeCharP (tape->checkpointFilename) ;
+  freeCharP (tape->peerName) ;
              
   removeTapeGlobally (tape) ;
   
@@ -631,6 +634,8 @@ void setTapeDirectory (const char *newDir)
   ASSERT (tapeDirectory == NULL) ;
 
   tapeDirectory = strdup (newDir) ;
+
+  addPointerFreedOnExit (tapeDirectory) ;
 }
 
 
@@ -638,8 +643,11 @@ void setTapeDirectory (const char *newDir)
 const char *getTapeDirectory ()
 {
   if (tapeDirectory == NULL)
-    tapeDirectory = strdup (TAPE_DIRECTORY) ;
-  
+    {
+      tapeDirectory = strdup (TAPE_DIRECTORY) ;
+      addPointerFreedOnExit (tapeDirectory) ;
+    }
+
   return tapeDirectory ;
 }
 
@@ -700,6 +708,12 @@ static void removeTapeGlobally (Tape tape)
     activeTapes [i] = activeTapes [i + 1] ;
 
   activeTapes [--activeTapeIdx] = NULL ;
+
+  if (activeTapeIdx == 0)
+    {
+      FREE (activeTapes) ;
+      activeTapes = NULL ;
+    }
 }
 
 
